/*---------------------------------------------------------------------------
Program :	SysTray.cpp	Version 1.0
		TSysTray Component for BCB1
Author :	Michel Leunen
Email :		michel@leunen.com
Date :		21/07/1998
Description :	Component for Borland C++ Builder V1 that places an icon in
		the Systray (near the clock).
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include <windows.h>
#include "SysTray.h"
//---------------------------------------------------------------------------
static inline TSysTray *ValidCtrCheck()
{
	return new TSysTray(NULL);
}
//---------------------------------------------------------------------------
__fastcall TSysTray::TSysTray(TComponent* Owner)
	: TComponent(Owner)
{
    
	FTrayHint="SysTray Hint";
    FIcon=new Graphics::TIcon;
    if(dynamic_cast<TWinControl *>(Owner))
    {
    	TWinControl *WinCtl=dynamic_cast<TWinControl *>(Owner);
    	//WndHandle=window handle that receives the notification message
    	WndHandle=WinCtl->Handle;
    	//defproc pointer to the default
    	DefProc=reinterpret_cast<void*>(GetWindowLong(WndHandle,GWL_WNDPROC));
    	//Assign WndProcInstance to new WndProc
    	WndProcInstance=MakeObjectInstance(SubWndProc);
    	SetWindowLong(WndHandle,GWL_WNDPROC,reinterpret_cast<long>(WndProcInstance));
    }
}
//---------------------------------------------------------------------------
__fastcall TSysTray::~TSysTray()
{
	delete FIcon;
	if(WndHandle!=NULL)
    	{
        //WndProc assigned to old WndProc
    	SetWindowLong(WndHandle,GWL_WNDPROC,reinterpret_cast<long>(DefProc));
        FreeObjectInstance(WndProcInstance);
    	}
}
//---------------------------------------------------------------------------
namespace Systray
{
	void __fastcall Register()
	{
		TComponentClass classes[1] = {__classid(TSysTray)};
		RegisterComponents("Leunen", classes, 0);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::SetIcon(TIcon *AIcon)
{
	FIcon->Assign(AIcon);
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::SetPopupMenu(TPopupMenu *APopupMenu)
{
	FPopupMenu=APopupMenu;
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::SetTrayHint(AnsiString ATrayHint)
{
	FTrayHint=ATrayHint;
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::AddIcon() 			// Add Icon to the tray
{
	NOTIFYICONDATA 	TNID;

    	TNID.cbSize=sizeof(NOTIFYICONDATA);
    	TNID.hWnd=WndHandle;
    	TNID.uID=ICON_ID;
    	TNID.uFlags=NIF_MESSAGE|NIF_ICON|NIF_TIP;
    	TNID.uCallbackMessage=TWMNOTIFYICON;
    	TNID.hIcon=FIcon->Handle;
    	strcpy(TNID.szTip,FTrayHint.c_str());
    	Shell_NotifyIcon(NIM_ADD,&TNID);
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::RemoveIcon()        	// Remove Icon from the tray
{
	NOTIFYICONDATA 	TNID;

	TNID.cbSize=sizeof(NOTIFYICONDATA);
	TNID.hWnd=WndHandle;
	TNID.uID=ICON_ID;
	Shell_NotifyIcon(NIM_DELETE,&TNID);
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::ModifyIcon()			// Change Icon or Hint
{
	NOTIFYICONDATA 	TNID;

	TNID.cbSize=sizeof(NOTIFYICONDATA);
	TNID.hWnd=WndHandle;
	TNID.uID=ICON_ID;
	TNID.uFlags=NIF_MESSAGE|NIF_ICON|NIF_TIP;
	TNID.uCallbackMessage=TWMNOTIFYICON;
	TNID.hIcon=FIcon->Handle;
	strcpy(TNID.szTip,FTrayHint.c_str());
	Shell_NotifyIcon(NIM_MODIFY,&TNID);
}
//---------------------------------------------------------------------------
void __fastcall TSysTray::SubWndProc(Messages::TMessage &Message)
{
    if(Message.Msg==TWMNOTIFYICON)
    {
    	UINT		MessageID;
    	UINT		MouseMessage;

    	MessageID=Message.WParam;
    	MouseMessage=(UINT)Message.LParam;
    	if(MouseMessage==WM_RBUTTONDOWN)
    	{
    		switch(MessageID)
        	{
        		case ICON_ID:
            		POINT pt;
            		GetCursorPos(&pt);
            		SetForegroundWindow(WndHandle);
            		FPopupMenu->Popup(pt.x,pt.y);
			PostMessage(WndHandle,WM_NULL,0,0);
            		break;
        	}
    	}
    	if(MouseMessage==WM_LBUTTONDBLCLK)
    	{
    		switch(MessageID)
        	{
        		case ICON_ID:
                	//execute method onClick of the first element in Popupmenu
                	FPopupMenu->DispatchPopup(FPopupMenu->Items->Items[0]->Handle);
            		break;
        	}
    	}
    }
    //send other messages to old WndProc
    Message.Result=CallWindowProc(reinterpret_cast<FARPROC>(DefProc),
    							  WndHandle,Message.Msg,Message.WParam,
                                  Message.LParam);
}
//---------------------------------------------------------------------------
